// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace ProjectExplorer { class Project; }

namespace CMakeProjectManager::Internal {

void setupCMakeInstallStep();
bool hasInstallDeployPreset(ProjectExplorer::Project *project);

} // CMakeProjectManager::Internal
