// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace AcpClient::Constants {

const char ACTION_ID[] = "AcpClient.Action";
const char MENU_ID[] = "AcpClient.Menu";
const char SHOW_CHAT_ACTION_ID[] = "AcpClient.ShowChat";
const char OUTPUT_PANE_ID[] = "AcpClient.ChatPanel";
const char INSPECT_ACTION_ID[] = "AcpClient.Inspect";
const char SHOW_CHAT_SIDEPANEL_ACTION_ID[] = "AcpClient.ShowChatSidePanel";

} // namespace AcpClient::Constants
