// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace Subversion::Internal {

const char FileUntrackedC  = '?';
const char FileAddedC      = 'A';
const char FileConflictedC = 'C';
const char FileDeletedC    = 'D';
const char FileModifiedC   = 'M';

} // SubVersion::Internal
