// Copyright (C) 2026 Sebastian Mosiej (sebastian.mosiej@wp.pl)
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace ZenMode::Constants {

const char DISTRACTION_FREE_ACTION_ID[] = "ZenMode.ToggleDistractionFreeMode";
const char ZEN_MODE_ACTION_ID[] = "ZenMode.ToggleZenMode";
const char MENU_ID[] = "ZenMode.Menu";

} // namespace ZenMode::Constants
